@@ -1,16 +1,28 @@
-/*
- * To change this template, choose Tools | Templates
- * and open the template in the editor.
- */
 package org.debian.maven.repo;
 
+/*
+ * Copyright 2009 Ludovic Claude.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 import java.util.*;
 
 /**
  *
  * @author Ludovic Claude <ludovicc@users.sourceforge.net>
  */
-public class Dependency {
+public class Dependency implements Comparable, Cloneable {
 
     private String groupId;
     private String artifactId;
@@ -19,6 +31,7 @@ public class Dependency {
     private boolean optional;
     private String scope;
     private String classifier;
+    private String relativePath;
 
     public Dependency(String groupId, String artifactId, String type, String version) {
         this.groupId = groupId;
@@ -30,7 +43,7 @@ public class Dependency {
     }
 
     public Dependency(String groupId, String artifactId, String type, String version, String scope, boolean optional,
-                      String classifier) {
+                      String classifier, String relativePath) {
         this.groupId = groupId;
         this.artifactId = artifactId;
         this.type = type;
@@ -38,6 +51,18 @@ public class Dependency {
         this.scope = scope;
         this.optional = optional;
         this.classifier = (classifier == null) ? "" : classifier;
+        this.relativePath = relativePath;
+    }
+
+    public Dependency(Dependency dependency) {
+        this.groupId = dependency.groupId;
+        this.artifactId = dependency.artifactId;
+        this.type = dependency.type;
+        this.version = dependency.version;
+        this.scope = dependency.scope;
+        this.optional = dependency.optional;
+        this.classifier = dependency.classifier;
+        this.relativePath = dependency.relativePath;
     }
 
     public String getArtifactId() {
@@ -95,6 +120,14 @@ public class Dependency {
     public void setClassifier(String classifier) {
         this.classifier = (classifier == null) ? "" : classifier;
     }
+    
+    public String getRelativePath() {
+        return relativePath;
+    }
+
+    public void setRelativePath(String relativePath) {
+        this.relativePath = relativePath;
+    }
 
     public boolean equals(Object obj) {
         if (obj == null) {
@@ -110,7 +143,7 @@ public class Dependency {
         if ((this.artifactId == null) ? (other.artifactId != null) : !this.artifactId.equals(other.artifactId)) {
             return false;
         }
-        if ((this.type == null) ? (other.type != null) : !this.type.equals(other.type)) {
+        if (!isTypeEquals(this.type, other.type)) {
             return false;
         }
         if ((this.version == null) ? (other.version != null) : !this.version.equals(other.version)) {
@@ -137,7 +170,7 @@ public class Dependency {
         if ((this.artifactId == null) ? (other.artifactId != null) : !this.artifactId.equals(other.artifactId)) {
             return false;
         }
-        if ((this.type == null) ? (other.type != null) : !this.type.equals(other.type)) {
+        if (!isTypeEquals(this.type, other.type)) {
             return false;
         }
         // Classifier is still important here as it can influence greatly the contents of the artifact (a source artifact is very different from a normal artifact)
@@ -151,7 +184,7 @@ public class Dependency {
         int hash = 7;
         hash = 31 * hash + (this.groupId != null ? this.groupId.hashCode() : 0);
         hash = 31 * hash + (this.artifactId != null ? this.artifactId.hashCode() : 0);
-        hash = 31 * hash + (this.type != null ? this.type.hashCode() : 0);
+        // ignore type
         hash = 31 * hash + (this.version != null ? this.version.hashCode() : 0);
         return hash;
     }
@@ -160,6 +193,39 @@ public class Dependency {
         return groupId + ":" + artifactId + ":" + type + ":" + version;
     }
 
+    public int compareTo(Object o) {
+        Dependency dependency = (Dependency) o; 
+        if (this.groupId != dependency.groupId) {
+            if (this.groupId == null) return -1;
+            if (! this.groupId.equals(dependency.groupId)) {
+                return this.groupId.compareTo(dependency.groupId);
+            }
+        }
+        if (this.artifactId != dependency.artifactId) {
+            if (this.artifactId == null) return -1;
+            if (! this.artifactId.equals(dependency.artifactId)) {
+                return this.artifactId.compareTo(dependency.artifactId);
+            }
+        }
+        // ignore type
+        if (this.version != dependency.version) {
+            if (this.version == null) return -1;
+            if (! this.version.equals(dependency.version)) {
+                return this.version.compareTo(dependency.version);
+            }
+        }
+        return 0;
+    }
+
+    public Object clone() {
+        try {
+            return super.clone();
+        } catch (CloneNotSupportedException e) {
+            e.printStackTrace();
+            return null;
+        }
+    }
+
     public Dependency applyRules(Collection rules) {
         for (Iterator i = rules.iterator(); i.hasNext();) {
             DependencyRule rule = (DependencyRule) i.next();
@@ -167,7 +233,7 @@ public class Dependency {
                 return rule.apply(this);
             }
         }
-        return this;
+        return new Dependency(this);
     }
 
     public DependencyRule findMatchingRule(Collection rules) {
@@ -200,9 +266,20 @@ public class Dependency {
         for (Iterator i = dependencies.iterator(); i.hasNext();) {
             Dependency dependency = (Dependency) i.next();
             if (dependency.findMatchingRule(ignoreRules) == null) {
-                result.add(dependency);
+                result.add(new Dependency(dependency));
             }
         }
         return result;
     }
+
+    public static boolean isTypeEquals(String type1, String type2) {
+        if (type1 != null && ("jar".equals(type1) || "bundle".equals(type1))) {
+            return ("jar".equals(type2) || "bundle".equals(type2));
+        }
+        if (type1 != null) {
+            return type1.equals(type2);
+        }
+        return type2 == null;
+    }
+
 }
