@@ -1,5 +1,21 @@
 package org.debian.maven.repo;
 
+/*
+ * Copyright 2009 Ludovic Claude.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
@@ -23,8 +39,9 @@ public class POMCleaner extends POMTransformer {
                 "distributionManagement", "profiles", "ciManagement", "prerequisites",
                 "repositories", "pluginRepositories", "reports", "reporting", "modelVersion",
                 "parent"});
-
     private boolean keepAllElements = false;
+    private boolean isPOM = false;
+    private Collection keepElements = new ArrayList();
 
     public POMCleaner() {
     }
@@ -37,8 +54,13 @@ public class POMCleaner extends POMTransformer {
         this.keepAllElements = keepAllElements;
     }
 
+    public void addElementToKeep(String element) {
+        keepElements.add(element);
+    }
+
     public void cleanPom(File originalPom, File targetPom, File pomProperties,
-            boolean noParent, boolean keepPomVersion, String setVersion, String debianPackage) {
+            boolean noParent, boolean hasPackageVersion, boolean keepPomVersion,
+            boolean keepParentVersion, String setVersion, String debianPackage) {
 
         if (targetPom.getParentFile() != null) {
             targetPom.getParentFile().mkdirs();
@@ -48,7 +70,7 @@ public class POMCleaner extends POMTransformer {
         }
 
         try {
-            POMInfo info = transformPom(originalPom, targetPom, noParent, keepPomVersion, setVersion, debianPackage);
+            POMInfo info = transformPom(originalPom, targetPom, noParent, hasPackageVersion, keepPomVersion, keepParentVersion, setVersion, debianPackage);
 
             Properties pomProps = new Properties();
             pomProps.put("groupId", info.getThisPom().getGroupId());
@@ -75,14 +97,32 @@ public class POMCleaner extends POMTransformer {
             addIgnoreRule(new DependencyRule(pom.getThisPom().getGroupId() + " "
                     + pom.getThisPom().getArtifactId() + " maven-plugin s/.*/"
                     + pom.getThisPom().getVersion() + "/"));
+        } else if (pom.getThisPom().getType().equals("pom")) {
+            isPOM = true;
         }
     }
 
-    protected boolean isWriteIgnoredElement(String element) {
-        if (keepAllElements) {
-            return super.isWriteIgnoredElement(element);
+    protected boolean isWriteIgnoredElement(String element, List path, Dependency dependency) {
+        boolean ignore = super.isWriteIgnoredElement(element, path, dependency);
+        if (keepAllElements || ignore) {
+            return ignore;
         }
-        return WRITE_IGNORED_ELEMENTS.contains(element);
+        if (keepElements.contains(element)) {
+            return false;
+        }
+        if (path.size() > 2 && keepElements.contains(path.get(1))) {
+            if ("version".equals(element)) {
+                String parent = (String) path.get(path.size() - 1);
+                if ("plugin".equals(parent)) {
+                    return true;
+                }
+            }
+            
+        }
+//        if (WRITE_IGNORED_ELEMENTS.contains(element)) {
+//            System.out.println("Write ignored " + element + " " + printPath(path) + " for " + dependency);
+//        }
+        return path.size() == 1 && WRITE_IGNORED_ELEMENTS.contains(element);
     }
 
     protected boolean acceptDependency(Dependency dependency, POMInfo info) {
@@ -92,22 +132,16 @@ public class POMCleaner extends POMTransformer {
         return "pom".equals(info.getThisPom().getType()) || !"test".equals(dependency.getScope());
     }
 
-    protected void writeDebianProperties(XMLStreamWriter writer, int inLevel, POMInfo original, POMInfo info, String debianPackage) throws XMLStreamException {
-        super.writeDebianProperties(writer, inLevel, original, info, debianPackage);
+    protected void createDebianProperties(POMInfo info, POMInfo original, String debianPackage, int inLevel) throws XMLStreamException {
+        super.createDebianProperties(info, original, debianPackage, inLevel);
         Map dependencyVersions = new TreeMap();
         for (Iterator i = original.getDependencies().iterator(); i.hasNext(); ) {
             Dependency dependency = (Dependency) i.next();
             if (dependency.getVersion() != null) {
-                dependencyVersions.put(dependency.getGroupId() + "." + dependency.getArtifactId(), dependency.getVersion());
+                String versionProperty = "debian." + dependency.getGroupId() + "." + dependency.getArtifactId() + ".originalVersion";
+                info.getProperties().put(versionProperty, dependency.getVersion());
             }
         }
-        for (Iterator i = dependencyVersions.entrySet().iterator(); i.hasNext(); ) {
-            Map.Entry depVer = (Map.Entry) i.next();
-            indent(writer, inLevel + 1);
-            writer.writeStartElement("debian." + depVer.getKey() + ".originalVersion");
-            writer.writeCharacters(depVer.getValue().toString());
-            writer.writeEndElement();
-        }
     }
 
     public static void main(String[] args) {
@@ -120,23 +154,39 @@ public class POMCleaner extends POMTransformer {
             System.out.println("  -o, --no-parent: don't inherit from a parent POM");
             System.out.println("  -p<package>, --package=<package>: name of the Debian package containing");
             System.out.println("    this library");
+            System.out.println("  -h --has-package-version: flag that indicates that this POM has the");
+            System.out.println("    same version as the package, this helps packagers of depending packages");
             System.out.println("  -r<rules>, --rules=<rules>: path to the file containing the");
             System.out.println("    extra rules to apply when cleaning the POM");
+            System.out.println("  -R<rule>, --extra-rule=<rule>: extra rule to apply when cleaning the POM");
+            System.out.println("    May occur multiple times, instead of or in addition to -r");
             System.out.println("  -u<rules>, --published-rules=<rules>: path to the file containing the");
             System.out.println("    extra rules to publish in the property debian.mavenRules in the cleaned POM");
+            System.out.println("  -U<rule>, --extra-published-rule=<rule>: extra rule to publish");
+            System.out.println("    May occur multiple times, instead of or in addition to -u");
             System.out.println("  -i<rules>, --ignore-rules=<rules>: path to the file containing the");
             System.out.println("    extra rules use to remove certain dependencies from the transformed POM");
             System.out.println("    This option can be repeated, in order to have multiple sets of");
             System.out.println("    dependencies to ignore, useful in situations such as when the Maven clean");
             System.out.println("    target requires more dependencies or plugins to ignore than the build target");
+            System.out.println("  -I<rule>, --extra-ignore-rule=<rule>: extra rule used to remove dependencies");
+            System.out.println("    from the transformed POM");
+            System.out.println("    May occur multiple times, instead of or in addition to -i");
             System.out.println("  --no-rules: don't apply any rules for converting versions, ");
             System.out.println("    do not even convert versions to the default 'debian' version");
+            System.out.println("  --no-publish-used-rule: don't publish the rule used to transform");
+            System.out.println("    a POM's own attributes in debian.mavenRules");
             System.out.println("  -e<version>, --set-version=<version>: set the version for the POM,");
             System.out.println("    do not use the version declared in the POM file.");
-            System.out.println("  --keep-pom-version: keep the original version of the POM but, ");
+            System.out.println("  --keep-pom-version: keep the original version of the POM but");
+            System.out.println("    convert all other versions in dependencies and plugins");
+            System.out.println("  --keep-parent-version: keep the version of the parent in the POM but");
             System.out.println("    convert all other versions in dependencies and plugins");
             System.out.println("  --keep-all-elements: keep all elements in the POM, do a version");
             System.out.println("    transformation only, don't delete the build and other elements.");
+            System.out.println("  --keep-elements=<elem1,elem2>: keep the elements listed here");
+            System.out.println("    even if they are normally removed by the clean operation.");
+            System.out.println("    Such elements are build,reports,reporting,prerequisites,profiles.");
             System.out.println("  -m<repo root>--maven-repo=<repo root>: location of the Maven repository,");
             System.out.println("    used to force the versions of the Maven plugins used in the current");
             System.out.println("    POM file with the versions found in the repository");
@@ -197,6 +247,11 @@ public class POMCleaner extends POMTransformer {
             System.out.println();
             System.out.println("  The default rule (* * * s/.*/debian/) replaces any version number with");
             System.out.println("  the 'debian' version and always applies last if there was no other matches.");
+            System.out.println();
+            System.out.println("  Any rules given on the command line (using -R/--extra-rule,");
+            System.out.println("  -U/--extra-published-rule or -I/--extra-ignore-rule) are applied");
+            System.out.println("  in the order given, in between the rules given in the rules file");
+            System.out.println("  (if any) and the default rules.");
             return;
         }
         POMCleaner cleaner = new POMCleaner();
@@ -204,14 +259,19 @@ public class POMCleaner extends POMTransformer {
         boolean verbose = false;
         boolean noParent = false;
         boolean noRules = false;
+        boolean hasPackageVersion = false;
         boolean keepPomVersion = false;
+        boolean keepParentVersion = false;
         boolean keepAllElements = false;
         String debianPackage = "";
         String setVersion = null;
         File rulesFile = null;
         File publishedRulesFile = null;
-        File ignoreRulesFile = null;
         File mavenRepo = null;
+        List rulesExtra = new ArrayList();
+        List publishedRulesExtra = new ArrayList();
+        List ignoreRulesExtra = new ArrayList();
+        List ignoreRulesFiles = new ArrayList();
         while (i < args.length && (args[i].trim().startsWith("-") || args[i].trim().length() == 0)) {
             String arg = args[i].trim();
             if ("--verbose".equals(arg) || "-v".equals(arg)) {
@@ -220,10 +280,22 @@ public class POMCleaner extends POMTransformer {
                 noParent = true;
             } else if ("--no-rules".equals(arg)) {
                 noRules = true;
+            } else if ("--no-publish-used-rule".equals(arg)) {
+                cleaner.setPublishUsedRule(false);
+            } else if ("--has-package-version".equals(arg) || "-h".equals(arg)) {
+                hasPackageVersion = true;
             } else if ("--keep-pom-version".equals(arg)) {
                 keepPomVersion = true;
+            } else if ("--keep-parent-version".equals(arg)) {
+                keepParentVersion = true;
             } else if ("--keep-all-elements".equals(arg)) {
                 keepAllElements = true;
+            } else if (arg.startsWith("--keep-elements=")) {
+                String keepElements = arg.substring("--keep-elements=".length());
+                StringTokenizer st = new StringTokenizer(keepElements, ",");
+                while (st.hasMoreTokens()) {
+                    cleaner.addElementToKeep(st.nextToken());
+                }
             } else if (arg.startsWith("-p")) {
                 debianPackage = arg.substring(2);
             } else if (arg.startsWith("--package=")) {
@@ -232,16 +304,28 @@ public class POMCleaner extends POMTransformer {
                 rulesFile = new File(arg.substring(2));
             } else if (arg.startsWith("--rules=")) {
                 rulesFile = new File(arg.substring("--rules=".length()));
+            } else if (arg.startsWith("-R")) {
+                rulesExtra.add(arg.substring(2));
+            } else if (arg.startsWith("--extra-rule=")) {
+                rulesExtra.add(arg.substring("--extra-rule=".length()));
             } else if (arg.startsWith("-u")) {
                 publishedRulesFile = new File(arg.substring(2));
             } else if (arg.startsWith("--published-rules=")) {
                 publishedRulesFile = new File(arg.substring("--published-rules=".length()));
+            } else if (arg.startsWith("-U")) {
+                publishedRulesExtra.add(arg.substring(2));
+            } else if (arg.startsWith("--extra-published-rule=")) {
+                publishedRulesExtra.add(arg.substring("--extra-published-rule=".length()));
             } else if (arg.startsWith("-i") || arg.startsWith("--ignore-rules=")) {
                 if (arg.startsWith("-i")) {
-                    ignoreRulesFile = new File(arg.substring(2));
+                    ignoreRulesFiles.add(new File(arg.substring(2)));
                 } else {
-                    ignoreRulesFile = new File(arg.substring("--ignore-rules=".length()));
+                    ignoreRulesFiles.add(new File(arg.substring("--ignore-rules=".length())));
                 }
+            } else if (arg.startsWith("-I")) {
+                ignoreRulesExtra.add(arg.substring(2));
+            } else if (arg.startsWith("--extra-ignore-rule=")) {
+                ignoreRulesExtra.add(arg.substring("--extra-ignore-rule=".length()));
             } else if (arg.startsWith("-e")) {
                 setVersion = arg.substring(2);
             } else if (arg.startsWith("--set-version=")) {
@@ -272,18 +356,32 @@ public class POMCleaner extends POMTransformer {
                     cleaner.getRules().setRulesFile(rulesFile);
                 }
             }
+            cleaner.getRules().addAll(rulesExtra);
 
-            if (ignoreRulesFile != null) {
-                if (ignoreRulesFile.exists()) {
-                    cleaner.getIgnoreRules().setRulesFile(ignoreRulesFile);
-                } else {
-                    System.err.println("Cannot find file: " + ignoreRulesFile);
+            if (!ignoreRulesFiles.isEmpty()) {
+                boolean firstIgnoreRule = true;
+                for (Iterator irf = ignoreRulesFiles.iterator(); irf.hasNext(); ) {
+                    File ignoreRulesFile = (File) irf.next();
+                    if (ignoreRulesFile.exists()) {
+                        if (firstIgnoreRule) {
+                            cleaner.getIgnoreRules().setRulesFile(ignoreRulesFile);
+                        } else {
+                            DependencyRuleSet tmpIgnoreRules = new DependencyRuleSet("Additional ignore rules");
+                            tmpIgnoreRules.setRulesFile(ignoreRulesFile);
+                            cleaner.getIgnoreRules().addAll(tmpIgnoreRules);
+                        }
+                        firstIgnoreRule = false;
+                    } else {
+                        System.err.println("Cannot find file: " + ignoreRulesFile);
+                    }
                 }
             }
+            cleaner.getIgnoreRules().addAll(ignoreRulesExtra);
 
             if (publishedRulesFile != null && publishedRulesFile.exists()) {
                 cleaner.getPublishedRules().setRulesFile(publishedRulesFile);
             }
+            cleaner.getPublishedRules().addAll(publishedRulesExtra);
 
             cleaner.addDefaultRules();
         }
@@ -295,8 +393,8 @@ public class POMCleaner extends POMTransformer {
         }
 
         cleaner.setKeepAllElements(keepAllElements);
-        cleaner.cleanPom(originalPom, targetPom, pomProperties, noParent,
-                keepPomVersion, setVersion, debianPackage);
+        cleaner.cleanPom(originalPom, targetPom, pomProperties, noParent, hasPackageVersion,
+                keepPomVersion, keepParentVersion, setVersion, debianPackage);
     }
 
     private static int inc(int i, String[] args) {
